#!/usr/bin/env python

from itertools import cycle
from math import ceil

def setaffinity(irq, coreidx, ncores, force=False):

    digits = int(ceil(1.*ncores/4))
    cpumask = '{0:x}'.format(1<<coreidx).zfill(digits)
    affinity = '/proc/irq/%s/smp_affinity' % irq

    with open(affinity,'r') as affinityfile:
        current_affinity = affinityfile.readline().rstrip('\n')

    with open(affinity,'w') as affinityfile:
        write = force or current_affinity != cpumask
        if write:
            print('Updating')
            affinityfile.write(cpumask)
    

interfaces=['eth0','eth1','eth2','eth3']

if __name__=='__main__':

    with open('/proc/interrupts','r') as f:
        irqmap = [l for l in f]
        
    robinirq = [l.split()[0].strip(':') for l in irqmap if 'robinnp' in l]
    robinirq.sort(key=int)
    
    netirqmap = {}
    for eth in interfaces:
        ## We need to append '-' to the interface name to be able
        ## to distinguish between e.g. dc1 and dc1s1
        netirqmap[eth] = [l.split()[0].strip(':') 
                          for l in irqmap if (eth+'-') in l]
        netirqmap[eth].sort(key=int)

    with open('/proc/cpuinfo','r') as cpuinfo:
        ncores = len([l for l in cpuinfo if 'processor	:' in l])

    ## Distribute Robin IRQ uniformerly an the available cores
    for idx,irq in enumerate(robinirq):
        coreidx = idx%ncores
        print coreidx, irq
        setaffinity(irq, coreidx, ncores)
        
        
    ## Group eth irqs into distinct subsets of the CPU cores
    ## Simple approximation, does not work for all PCs
    ## it works for the Gen3 PC model
    subsetsize = ncores/len(interfaces)
    
    #coresets = [(0,6,7),(1,2,8),(3,9,10),(4,5,11)]

    for idx,eth in enumerate(sorted(netirqmap.keys())):
        irqs = netirqmap[eth]
        startidx = idx*subsetsize
        cores = range(startidx, startidx + subsetsize)
        #cores = coresets[idx]
        for irq, coreidx in zip(irqs, cycle(cores)):
            print eth,irq,coreidx
            setaffinity(irq, coreidx, ncores)
            
